package com.futhark;

import net.minecraft.nbt.NbtCompound;
import net.minecraft.server.MinecraftServer;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.PersistentState;
import net.minecraft.world.PersistentStateManager;
import net.minecraft.world.World;

import java.util.HashMap;

public class StateSaverAndLoader extends PersistentState {

    public Integer totalNumberOfRunes = 0;
    public HashMap<BlockPos, RuneData> runes = new HashMap<>();

    @Override
    public NbtCompound writeNbt(NbtCompound nbt) {
        nbt.putInt("totalNumberOfRunes", totalNumberOfRunes);

        NbtCompound runesNbt = new NbtCompound();
        runes.forEach((pos, runeData) -> {
            NbtCompound runeNbt = new NbtCompound();

            runeNbt.putInt("type", runeData.type.asInt());

            runesNbt.put(Long.toString(pos.asLong()), runeNbt);
        });
        nbt.put("runes", runesNbt);

        return nbt;
    }

    public static StateSaverAndLoader createFromNbt(NbtCompound tag) {
        StateSaverAndLoader state = new StateSaverAndLoader();
        state.totalNumberOfRunes = tag.getInt("totalNumberOfRunes");

        NbtCompound runesNbt = tag.getCompound("runes");
        runesNbt.getKeys().forEach(key -> {
            long longPos = Long.parseLong(key);
            BlockPos pos = BlockPos.fromLong(longPos);

            NbtCompound runeNbt = runesNbt.getCompound(key);

            RuneData runeData = new RuneData();
            runeData.type = RuneType.fromInt(runeNbt.getInt("type"));
            runeData.pos = pos;

            state.runes.put(pos, runeData);
        });

        return state;
    }

    /**
     * This function gets the 'PersistentStateManager' and creates or returns the filled in 'StateSaveAndLoader'.
     * It does this by calling 'StateSaveAndLoader::createFromNbt' passing it the previously saved 'NbtCompound' we wrote in 'writeNbt'.
     */
    public static StateSaverAndLoader getServerState(MinecraftServer server) {
        PersistentStateManager persistentStateManager = server.getWorld(World.OVERWORLD).getPersistentStateManager();

        StateSaverAndLoader state = persistentStateManager.getOrCreate(
                StateSaverAndLoader::createFromNbt,
                StateSaverAndLoader::new,
                Futhark.MOD_ID
        );

        // If state is not marked dirty, when Minecraft closes, 'writeNbt' won't be called and therefore nothing will be saved.
        // Technically it's 'cleaner' if you only mark state as dirty when there was actually a change, but the vast majority
        // of mod writers are just going to be confused when their data isn't being saved, and so it's best just to 'markDirty' for them.
        // Besides, it's literally just setting a bool to true, and the only time there's a 'cost' is when the file is written to disk when
        // there were no actual change to any of the mods state (INCREDIBLY RARE).
        state.markDirty();

        return state;
    }
}